#include <iostream>
#include <occa.hpp>

int main(int argc, const char **argv) {

  int entries = 5;

  float *a  = new float[entries];
  float *b  = new float[entries];
  float *c = new float[entries];

  for (int i = 0; i < entries; ++i) {
    a[i]  = i;
    b[i]  = 1 - i;
    c[i] = 0;
  }

  //---[ Device setup with string flags ]-------------------

  occa::device device;
  
  device.setup("mode: 'Serial'");

  // device.setup("mode     : 'OpenMP', "
  //              "schedule : 'compact', "
  //              "chunk    : 10");

  // device.setup("mode        : 'OpenCL', "
  //              "platform_id : 0, "
  //              "device_id   : 1");

  // device.setup("mode      : 'CUDA', "
  //              "device_id : 0");
  //========================================================

  // Allocate memory on the device
  occa::memory o_a  = device.malloc(entries * sizeof(float));
  occa::memory o_b  = device.malloc(entries * sizeof(float));
  occa::memory o_c = device.malloc(entries * sizeof(float));

  // Copy memory to the device
  o_a.copyFrom(a);
  o_b.copyFrom(b);

  // introduce occa properties
  occa::properties props;
  props["defines/p_Nthreads"] = 16;
  props["compiler_flags"] = "-arch=sm_30";  // NVIDIA-specific
  
  // Compile the kernel at run-time
  occa::kernel addVectors =
    device.buildKernel("addVectors.okl","addVectors",props);
 
  // Launch device kernel
  // tic
  addVectors(entries, o_a, o_b, o_c);
  device.finish(); // global device synchronization  
  // toc
  


  // Copy result to the host
  o_c.copyTo(c);

  // Assert values
  for (int i = 0; i < 5; ++i) {
    std::cout << i << ": " << c[i] << '\n';
  }
  for (int i = 0; i < entries; ++i) {
    if (c[i] != (a[i] + b[i])) {
      throw 1;
    }
  }

  // Free host memory
  delete [] a;
  delete [] b;
  delete [] c;

  return 0;
}

